@@ -972,6 +972,34 @@ impl Connection {
         }
     }
 
+    pub fn kem_name(&self) -> Option<&str> {
+        let name_bytes = {
+            let name = unsafe { s2n_connection_get_kem_name(self.connection.as_ptr()) };
+            if name.is_null() {
+                return None;
+            }
+            name
+        };
+
+        let name_str = unsafe {
+            // SAFETY: The data is null terminated because it is declared as a C
+            //         string literal.
+            // SAFETY: kem_name has a static lifetime because it lives on a const
+            //         struct s2n_kem with file scope.
+            const_str!(name_bytes)
+        };
+
+        match name_str {
+            Ok("NONE") => None,
+            Ok(name) => Some(name),
+            Err(_) => {
+                // Unreachable: This would indicate a non-utf-8 string literal in
+                // the s2n-tls C codebase.
+                None
+            }
+        }
+    }
+
     pub fn selected_curve(&self) -> Result<&str, Error> {
         let curve = unsafe { s2n_connection_get_curve(self.connection.as_ptr()).into_result()? };
         unsafe {
@@ -1070,13 +1098,10 @@ impl Connection {
     // chain, so the lifetime is independent of the connection.
     pub fn peer_cert_chain(&self) -> Result<CertificateChain<'static>, Error> {
         unsafe {
-            let mut chain = CertificateChain::new()?;
-            s2n_connection_get_peer_cert_chain(
-                self.connection.as_ptr(),
-                chain.as_mut_ptr().as_ptr(),
-            )
-            .into_result()
-            .map(|_| ())?;
+            let mut chain = CertificateChain::allocate_owned()?;
+            s2n_connection_get_peer_cert_chain(self.connection.as_ptr(), chain.as_mut_ptr())
+                .into_result()
+                .map(|_| ())?;
             Ok(chain)
         }
     }
