@@ -12,6 +12,7 @@ mod tests {
     use alloc::sync::Arc;
     use core::sync::atomic::Ordering;
     use futures_test::task::{new_count_waker, noop_waker};
+    use security::Policy;
     use std::{fs, path::Path, pin::Pin, sync::atomic::AtomicUsize};
 
     #[test]
@@ -26,6 +27,34 @@ mod tests {
         assert!(TestPair::handshake_with_config(&config).is_ok());
     }
 
+    #[test]
+    fn kem_name_retrieval() -> Result<(), Error> {
+        // PQ isn't supported
+        {
+            let policy = Policy::from_version("20240501")?;
+            let config = build_config(&policy)?;
+            let mut pair = TestPair::from_config(&config);
+
+            // before negotiation, kem_name is none
+            assert!(pair.client.kem_name().is_none());
+
+            pair.handshake().unwrap();
+            assert!(pair.client.kem_name().is_none());
+        }
+
+        // PQ is supported
+        {
+            let policy = Policy::from_version("KMS-PQ-TLS-1-0-2020-07")?;
+            let config = build_config(&policy)?;
+            let mut pair = TestPair::from_config(&config);
+
+            pair.handshake().unwrap();
+            assert_eq!(pair.client.kem_name(), Some("kyber512r3"));
+        }
+
+        Ok(())
+    }
+
     #[test]
     fn default_config_and_clone_interaction() -> Result<(), Error> {
         let config = build_config(&security::DEFAULT_TLS13)?;
